package mcmultipart.block;

import java.util.Collection;
import java.util.UUID;

import mcmultipart.multipart.IMultipart;
import mcmultipart.multipart.IMultipartContainer;
import mcmultipart.multipart.ISlottedPart;
import mcmultipart.multipart.Multipart;
import mcmultipart.multipart.MultipartContainer;
import mcmultipart.multipart.PartSlot;
import net.minecraft.block.BlockContainer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.S35PacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ITickable;
import net.minecraft.world.World;

/**
 * A final class that extends {@link BlockContainer} and implements {@link IMultipartContainer}. Represents a TileEntity which can contain
 * any kind of multipart.<br/>
 * You do NOT need to extend this class for your multiparts to work. I repeat, you do NOT. You need to either extend {@link Multipart} or
 * implement {@link IMultipart}. If you only need microblock support, look into {@link BlockCoverable}.
 */
public final class TileMultipart extends TileEntity implements IMultipartContainer, ITickable {

    private MultipartContainer container;

    public TileMultipart(MultipartContainer container) {

        this.container = new MultipartContainer(this, container.canTurnIntoBlock(), container);
    }

    public TileMultipart() {

        this.container = new MultipartContainer(this, true);
    }

    @Override
    public World getWorldIn() {

        return func_145831_w();
    }

    @Override
    public BlockPos getPosIn() {

        return func_174877_v();
    }

    public MultipartContainer getPartContainer() {

        return container;
    }

    @Override
    public Collection<? extends IMultipart> getParts() {

        return container.getParts();
    }

    @Override
    public ISlottedPart getPartInSlot(PartSlot slot) {

        return container.getPartInSlot(slot);
    }

    @Override
    public boolean canAddPart(IMultipart part) {

        return container.canAddPart(part);
    }

    @Override
    public boolean canReplacePart(IMultipart oldPart, IMultipart newPart) {

        return container.canReplacePart(oldPart, newPart);
    }

    @Override
    public void addPart(IMultipart part) {

        container.addPart(part);
    }

    @Override
    public void removePart(IMultipart part) {

        container.removePart(part);
    }

    @Override
    public UUID getPartID(IMultipart part) {

        return container.getPartID(part);
    }

    @Override
    public IMultipart getPartFromID(UUID id) {

        return container.getPartFromID(id);
    }

    @Override
    public void addPart(UUID id, IMultipart part) {

        container.addPart(id, part);
    }

    @Override
    public void func_73660_a() {

        if (!getWorldIn().field_72995_K && getParts().isEmpty()) {
            getWorldIn().func_175698_g(getPosIn());
            return;
        }

        for (IMultipart part : getParts())
            if (part instanceof ITickable) ((ITickable) part).func_73660_a();
    }

    @Override
    public void onLoad() {

        super.onLoad();
        for (IMultipart part : getParts())
            part.onLoaded();
    }

    @Override
    public void onChunkUnload() {

        super.onChunkUnload();
        for (IMultipart part : getParts())
            part.onUnloaded();
    }

    @Override
    public void func_145841_b(NBTTagCompound compound) {

        super.func_145841_b(compound);
        container.writeToNBT(compound);
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {

        super.func_145839_a(compound);
        container.readFromNBT(compound);
    }

    @Override
    public S35PacketUpdateTileEntity func_145844_m() {

        NBTTagCompound tag = new NBTTagCompound();
        container.writeDescription(tag);
        return new S35PacketUpdateTileEntity(getPosIn(), func_145832_p(), tag);
    }

    @Override
    public void onDataPacket(NetworkManager net, S35PacketUpdateTileEntity pkt) {

        container.readDescription(pkt.func_148857_g());
    }

    @Override
    public boolean canRenderBreaking() {

        return true;
    }

    @Override
    public boolean shouldRenderInPass(int pass) {

        return true;
    }

    @Override
    public AxisAlignedBB getRenderBoundingBox() {

        AxisAlignedBB bounds = null;
        for (IMultipart part : getParts()) {
            AxisAlignedBB bb = part.getRenderBoundingBox();
            if (bb != null) {
                if (bounds == null) bounds = bb;
                else bounds = bounds.func_111270_a(bb);
            }
        }
        if (bounds == null) bounds = AxisAlignedBB.func_178781_a(0, 0, 0, 1, 1, 1);
        return bounds.func_72317_d(getPosIn().func_177958_n(), getPosIn().func_177956_o(), getPosIn().func_177952_p());
    }

}
